package com.ruoyi.project.production.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.security.LoginUser;
import com.ruoyi.framework.security.service.TokenService;
import com.ruoyi.project.production.controller.vo.SaiApplyQueryVO;
import com.ruoyi.project.system.domain.*;
import com.ruoyi.project.system.mapper.SysDeptMapper;
import com.ruoyi.project.system.mapper.SysRoleMapper;
import com.ruoyi.project.system.mapper.SysUserMapper;
import com.ruoyi.project.system.mapper.SysUserRoleMapper;
import com.ruoyi.project.system.service.ISysDictTypeService;
import com.ruoyi.project.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.production.mapper.TSaiApplyMapper;
import com.ruoyi.project.production.domain.TSaiApply;
import com.ruoyi.project.production.service.ITSaiApplyService;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;

/**
 * SAI开项管理Service业务层处理
 *
 * @author ruoyi
 * @date 2023-03-15
 */
@Service
public class TSaiApplyServiceImpl implements ITSaiApplyService
{
    @Autowired
    private TSaiApplyMapper tSaiApplyMapper;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private ISysDictTypeService iSysDictTypeService;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    /**
     * 查询SAI开项管理
     *
     * @param saiApplyId SAI开项管理ID
     * @return SAI开项管理
     */
    @Override
    public TSaiApply selectTSaiApplyById(Long saiApplyId)
    {
        TSaiApply tSaiApply = tSaiApplyMapper.selectTSaiApplyById(saiApplyId);
        if (tSaiApply.getApplicant() != null) {
            tSaiApply.setApplicantName(sysUserMapper.selectUserById(Long.parseLong(tSaiApply.getApplicant())).getNickName());
        }
        return tSaiApply;
    }

    /**
     * 查询SAI开项管理列表
     *
     * @param tSaiApply SAI开项管理
     * @return SAI开项管理
     */
    @Override
    public List<TSaiApply> selectTSaiApplyList(SaiApplyQueryVO tSaiApply)
    {
        List<TSaiApply> tSaiApplies = tSaiApplyMapper.selectTSaiApplyList(tSaiApply);
        List<SysDictData> staffUnitDict = iSysDictTypeService.selectDictDataByType("STAFF_UNIT");
        for (TSaiApply saiApply : tSaiApplies) {
            String applicantDept = saiApply.getApplicantDept();
            String applicant = saiApply.getApplicant();
            String handler = saiApply.getHandler();
            String executor = saiApply.getExecutor();
            if (applicantDept != null) {
                for (SysDictData sysDictData : staffUnitDict) {
                    if (sysDictData.getDictValue().equals(applicantDept)) {
                        saiApply.setApplicantDeptName(sysDictData.getDictLabel());
                    }
                }
            }
            if (applicant != null) {
                saiApply.setApplicantName(sysUserMapper.selectUserById(Long.parseLong(applicant)).getNickName());
            }
            if (handler != null) {
                if (handler.indexOf(",") != -1) {
                    String[] split = handler.split(",");
                    String handlerName = "";
                    for (int i = 0; i < split.length; i++) {
                        handlerName += sysUserMapper.selectUserById(Long.parseLong(split[i])).getNickName();
                        if (i < split.length - 1) {
                            handlerName += ",";
                        }
                    }
                    saiApply.setHandlerName(handlerName);
                } else {
                    saiApply.setHandlerName(sysUserMapper.selectUserById(Long.parseLong(handler)).getNickName());
                }
            }
            if (executor != null) {
                saiApply.setExecutorName(sysUserMapper.selectUserById(Long.parseLong(executor)).getNickName());
            }
        }
        boolean specialRole = this.isSpecialRole();
        if (specialRole) {  // 当前登录用户是特殊角色
            for (TSaiApply saiApply : tSaiApplies) {
                Long applyStatus = saiApply.getApplyStatus();
                if (applyStatus.toString().equals("5")) {   // 申请状态为“已取消”的数据
                    tSaiApplies.remove(saiApply);   // 从列表中移除
                }
            }
        }
        return tSaiApplies;
    }

    /**
     * 当前登录用户是否为特殊角色（供SAI功能展示使用）
     *
     * @return 是否为特殊角色
     */
    private boolean isSpecialRole() {
        boolean isSpecialRole = false;
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        SysUser user = loginUser.getUser();
        List<SysRole> roles = user.getRoles();
        for (SysRole role : roles) {
            Long roleId = role.getRoleId();
            if (roleId.toString().equals("23140")) {
                isSpecialRole = true;
            }
        }
        return isSpecialRole;
    }

    /**
     * 新增SAI开项管理
     *
     * @param tSaiApply SAI开项管理
     * @return 结果
     */
    @Override
    public int insertTSaiApply(TSaiApply tSaiApply)
    {
        tSaiApply.setCreateTime(DateUtils.getNowDate());
        return tSaiApplyMapper.insertTSaiApply(tSaiApply);
    }

    /**
     * 修改SAI开项管理
     *
     * @param tSaiApply SAI开项管理
     * @return 结果
     */
    @Override
    public int updateTSaiApply(TSaiApply tSaiApply)
    {
        tSaiApply.setUpdateTime(DateUtils.getNowDate());
        return tSaiApplyMapper.updateTSaiApply(tSaiApply);
    }

    /**
     * 批量删除SAI开项管理
     *
     * @param saiApplyIds 需要删除的SAI开项管理ID
     * @return 结果
     */
    @Override
    public int deleteTSaiApplyByIds(Long[] saiApplyIds)
    {
        return tSaiApplyMapper.deleteTSaiApplyByIds(saiApplyIds);
    }

    /**
     * 删除SAI开项管理信息
     *
     * @param saiApplyId SAI开项管理ID
     * @return 结果
     */
    @Override
    public int deleteTSaiApplyById(Long saiApplyId)
    {
        return tSaiApplyMapper.deleteTSaiApplyById(saiApplyId);
    }
}
